/* Copyright (C) 1995, Sun Microsystems, Inc. */

/*
 * FUNCTION
 *      vdk_c_fir_16 - finite impulse response filter
 *
 * SYNOPSIS
 *      void vdk_c_fir_16 (vis_s16 *src, vis_s16 *dst, int dlen, 
 *                         vis_s16 *fir, int flen);
 *
 * ARGUMENT
 *      src     printer to first sample of source data
 *      dst     printer to first sample of destination data
 *      dlen    length of destination data
 *      fir     coefficients of FIR filter
 *      flen    length of FIR filter
 *
 * DESCRIPTION
 *              flen-1
 *      dst[n] = SUM{fir[k]*src[n+k]}, 0 <= n < dlen
 *               k=0
 */

#include <stdlib.h>
#include "vis_types.h"

/***************************************************************/

void vdk_c_fir_16 (vis_s16 *src, vis_s16 *dst, int dlen, 
                   vis_s16 *fir, int flen)
{
  int     n, k, num;    /* loop variables */
  vis_s32 tmp;

  for (n = 0; n < dlen; n ++) {
    tmp = 0;
    for (k = 0; k < flen; k ++)
      tmp += fir[k] * src[n+k];
    dst[n] = (vis_s16) (tmp >> 16);
  }
}

/***************************************************************/
