defstar {
  name { StrictPure }
  domain { FSM }
  derivedfrom { FSMStateStar }
  desc { 
A star placed in the graph to represent the state of a finite state machine
with pure signals and evaluated in strict manner.
  }
  version {@(#)FSMStrictPure.pl	1.3 12/09/97}
  author { Bilung Lee }
  copyright {
Copyright (c) 1997-1997 The Regents of the University of California.
All rights reserved.
See the file ~ptolemy/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
  }
  location { FSM main library }

  ccinclude { "FSMScheduler.h" }
  ccinclude { "InfString.h" }
  ccinclude { "utilities.h" }

  protected {
    // Input buffers of the inputs and internal events of the master FSM.
    int* allInBuf;
    // Output buffers of the outputs and internal events of the master FSM.
    int* allOutBuf;
  }

  setup {
    FSMStateStar::setup();

    // Setup allInBuf/allOutBuf;
    SPureSched *sched = (SPureSched *)scheduler();
    allInBuf  = sched->allInBuf();
    allOutBuf = sched->allOutBuf();
  }   // end of setup()

  go {
  }

  method {
    name { execAction }
    type { int }
    arglist { "(int actNum)" }
    access { public }
    code {
      int numOutputs = parsedAct[actNum].size();
      for (int i=0; i<numOutputs; i++)
	allOutBuf[parsedAct[actNum][i]] = 1;

      return TRUE;
    }
  }

  method {
    name { execSlave }
    type { int }
    arglist { "(int curEntryType)" }
    access { public }
    code {
      ///////////////////////////////////////////////////////////////////
      // If the slave process exists, then execute it.

      int i;
      if (slave != NULL) {
	// Grab the input.
	Particle* pp;
	for (i=0; i<slave->inNmList.numPieces(); i++) {
	    pp = new IntParticle(allInBuf[slave->inGeoIndx[i]]);
	    (slave->inGeo[i])->put(pp);
	}

	if (curEntryType == 1) {
	  // Enter internal machine by "Default" (initial state).
	  slave->worm->begin();
	}

	// Do the data processing.
	if (!(slave->worm->run())) return FALSE;

	for (i=0; i<slave->outNmList.numPieces(); i++) {
	  // Put the outputs to the output buffers.
	  // Question: Do we still modify the output buffer even when
	  // the output of the slave process is absent (0)?
	  // So far, YES. It seems that we allow a system that can
	  // make present signals disappear.
	  pp = slave->outGeo[i]->get();
	  allOutBuf[slave->outGeoIndx[i]] = int(*pp);

	  // If the output is an internal event of the master FSM,
	  // then it is also put to the input buffers.
	  if (slave->outGeoIndx[i] < internalEventNm->numPieces()) {
	    // The output is an internal event, so it will affect input.
	    allInBuf[slave->outGeoIndx[i]] = int(*pp);
	  }
	}

      }  // end of if (slave != NULL)

      return TRUE;
    }  // end of code
  }    // end of execSlave()

  method {
    name { printAllInBuf }
    type { void }
    access { public }
    code {
      FSMScheduler* sched = (FSMScheduler*)scheduler();
      int num = sched->inPorts()->numberPorts()
	        + internalEventNm->numPieces();
      for (int i=0; i<num; i++) {
	printf("myAllInBuf[%d] = %d\n",i,allInBuf[i]);
      }
    }
  }

  method {
    name { printAllOutBuf }
    type { void }
    access { public }
    code {
      FSMScheduler* sched = (FSMScheduler*)scheduler();
      int num = sched->outPorts()->numberPorts()
	        +internalEventNm->numPieces();
      for (int i=0; i<num; i++) {
	printf("myAllOutBuf[%d] = %d\n",i,allOutBuf[i]);
      }
    }
  }

}     // end of defstar
