/* A Parameter is a container for a token.

 Copyright (c) 1998 The Regents of the University of California.
 All rights reserved.
 Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.
 
                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY

@ProposedRating Red (nsmyth@eecs.berkeley.edu)

*/

package pt.data;

import pt.kernel.*;
import pt.data.parser.*;
import java.util.Observable;

//////////////////////////////////////////////////////////////////////////
//// Param
/** 
 * A Param is a container for a token. Params are relatively 
 * simple oblects, with any complicated processing being passed to a 
 * Controller object. The Controller object handles issues such as parsing 
 * strings and handling dependencies on other Params. If another Object (eg
 * Controller) wants to Observe this Param, it must register its interest 
 * with the TokenPublisher associated with the contained Token.
 * Each Param can be associated with at most one Controller. 
 * 
 * @author Neil Smyth
 * @version @(#)Param.java	1.13 04/24/98
 * @see pt.data.parser.PtSimpleParser 
 * @see pt.data.Token 
*/
public abstract class Param extends NamedObj {
    
    /** Construct a Param with the given Token in the workspace of its 
     *  container with the given name. 
     *  If the name argument is null, then the name is set to the empty string.
     *  Increment the version of the workspace.
     *  @param container The NamedObj which contains this parameter
     *  @param name The name of this object, given to the superclass
     *  @param token The Token contained by this Param
     *  constructor.
     */
     protected Param(NamedObj container, String name, Token token) {
         super(container.workspace(), name);
         _container = container;
         _token = token;
     }

     /** Construct a Param from the given String in the workspace of its 
     *  container with the given name. 
     *  The objects name is set to the empty string.
     *  Increment the version of the workspace.
     *  @param container The NamedObj which contains this parameter
     *  @param value The String to be parsed to a Token
     */
     protected Param(NamedObj container, String name, String value) {
         super(container.workspace(), name);
         _container = container;
         // now need to sync as could depend on other Params
         synchronized(workspace()) {
             //_controller = new PtSimpleParser();
             //FIXME: not finished...
         }
     }


    //////////////////////////////////////////////////////////////////////////
    ////                         public methods                           ////
    

    /** Get the Controller Object associated with this Param.
     * If this Param does not already have a Controller object associated
     * with it, create one.  
     * @return Controller 
     */	
    /*public PtSimpleParser getController() {
        synchronized(workspace()) {
            if ( _controller == null ) {
                _controller = new PtSimpleParser();
            }
        }
    	return _controller;
    }*/

    /** Reset the current value of the state to the one that was given
     * to it when it was created.
     */	
    public abstract void reset();
    
    /** Return the number of elements in the parameter. Scalar parameters
     * will return one; array parameters will return the number of elements
     * in the array.
     * FIXME : Do we want this method? Same applies to reset & isArray
     * @return The number of elements.
     */	
    public int size() {
        return 1;
    }

    /** Get a string representation of the current parameter value.
     * @return A String representing the current value.
     */	
    public abstract String toString();

    //////////////////////////////////////////////////////////////////////////
    ////                         protected methods                        ////

    
    //////////////////////////////////////////////////////////////////////////
    ////                         protected variables                      ////

    //////////////////////////////////////////////////////////////////////////
    ////                         private methods                          ////

    //////////////////////////////////////////////////////////////////////////
    ////                         private variables                        ////

    //private PtSimpleParser _controller;
    private NamedObj _container;
    private Token _token;
    private Class _constrainType;

}
